'' =================================================================================================
''
''   File....... jm_dual_servo.spin
''   Purpose.... 
''   Author..... Jon "JonnyMac" McPhalen (aka Jon Williams)
''               Copyright (c) 2010 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 23 JUN 2010
''
'' =================================================================================================


con

  _clkmode = xtal1 + pll16x                                     ' setup for 80MHz
  _xinfreq = 5_000_000 

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq               ' ticks per second
  MS_001   = CLK_FREQ / 1_000                                   ' ticks per millisecond
  US_001   = CLK_FREQ / 1_000_000                               ' ticks per microsecond
  

con

  SVO1 = 0
  SVO2 = 1

  SVO_MIN =  400                                                ' 0.4ms (Hitec)
  SVO_CTR = 1500                                                ' 1.5ms
  SVO_MAX = 2400                                                ' 2.4ms (Hitec)  
  

con 

  #1, HOME, #8, BKSP, TAB, LF, CLREOL, CLRDN, CR, #16, CLS      ' PST formmatting control


obj

  term : "fullduplexserial"


var

  long  servo1                                                  ' servo 1 position (us)
  long  servo2                                                  ' servo 2 position (us)
  long  servostack[16]                                          ' stack for servo cog


dat

Menu            byte    CLS
                byte    "[1] Servo 1 Center", CR
                byte    "[2] Servo 1 Min   ", CR
                byte    "[3] Serov 1 Max   ", 0


pub main  | c

  servo1 := SVO_CTR                                             ' initialize startin positions
  servo2 := SVO_CTR                                             
  cognew(servos, @servostack)                                   ' start dual servo driver

  term.start(31, 30, %0000, 115_200)                            ' start terminal
  pause(5)
  term.str(@Menu)
  
  repeat
    c := term.rx                                                ' wait for command
    case c                                                      ' reposition servo if valid cmd
      "1" : servo1 := SVO_CTR
      "2" : servo1 := SVO_MIN
      "3" : servo1 := SVO_MAX
  
 
pub pause(ms) | t

  t := cnt - 1088                                               ' sync with system counter
  repeat (ms #> 0)
    waitcnt(t += MS_001)


pub servos | t, pos

'' Background driver for two (or more servos, up to eight max)
'' -- must be launched into own cog (use cognew)

  dira[SVO1] := 1                                               ' make servo pins outputs/low
  outa[SVO1] := 0
  dira[SVO2] := 1
  outa[SVO2] := 0

  t := cnt                                                      ' sync point for 20ms frame
  repeat
    pos := (SVO_MIN #> servo1 <# SVO_MAX) * US_001 - 989        ' to ticks, adjust for overhead
    outa[SVO1] := 1                                             ' create servo 1 pulse
    waitcnt(pos + cnt)
    outa[SVO1] := 0
    
    pos := (SVO_MIN #> servo2 <# SVO_MAX) * US_001 - 989
    outa[SVO2] := 1                                       
    waitcnt(pos + cnt)
    outa[SVO2] := 0

    waitcnt(t += constant(20 * MS_001))                         ' wait for end of 20ms frame

                      
dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}  